/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */

package ca.uvic.csr.shrimp.adapter;

import java.awt.Frame;

import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.ChildParentChooserWindow;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.usercontrols.DefaultProjectAction;
import ca.uvic.csr.shrimp.util.ShrimpUtils;

/**
 * This adapter displays a dialog allowing the user to choose a new hierarchy to
 * base the shrimp view on, and choose whether or not to show composites in the shrimp view.
 * 
 * @author Casey Best, Rob Lintern
 */
public class ChangeCPRelAdapter extends DefaultProjectAction {
	
	public static final String ACTION_NAME = "Change Hierarchy...";
	
	public ChangeCPRelAdapter(ShrimpProject project) {
		super(ACTION_NAME, project);
	}
	
	public void startAction() {
		Frame parentFrame = ApplicationAccessor.getApplication().getParentFrame();
		try {
			ShrimpProject project = getProject();
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);	
			DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean (ShrimpTool.DISPLAY_BEAN);
			ChildParentChooserWindow chooser = new ChildParentChooserWindow(parentFrame, displayBean, dataBean);
			if (chooser.accepted()) {
			    String[] chosenCprels = chooser.getCprels();
                boolean chosenInverted = chooser.isInverted();
			    if (ShrimpUtils.shouldChangeCprels(chosenCprels, chosenInverted, dataBean.getDefaultCprels(), dataBean.getDefaultCprelsInverted())) {
			        if (ShrimpUtils.hasRootArtifacts(chosenCprels, chosenInverted, dataBean)) {
			            if (ShrimpUtils.shouldShowManyRoots(chosenCprels, chosenInverted, dataBean)) {
							boolean dataBeanFiringEvents = dataBean.isFiringEvents();
							dataBean.setFiringEvents(false);
							shrimpView.setCprels (chosenCprels, chosenCprels.length > 0 && chosenInverted, true);
							shrimpView.addDefaultRootNodes(true);
							//shrimpView.navigateToObject(displayBean.getDataDisplayBridge().getRootNodes());
							dataBean.setFiringEvents(dataBeanFiringEvents);
			            }
				    }
			    }
			} 		
		} catch (ShrimpToolNotFoundException stnfe) {
			stnfe.printStackTrace();
		} catch (BeanNotFoundException bnfe) {
		    bnfe.printStackTrace();
		}	
	}
	
}
