/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.options;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.eclipse.mylar.zest.layouts.algorithms.MotionLayoutAlgorithm;

import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.layout.Layout;
import ca.uvic.csr.shrimp.DisplayBean.layout.MotionLayout;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.ActionManager.CheckBoxAction;
import ca.uvic.csr.shrimp.gui.QueryView.QueryView;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.util.JIntegerTextField;

/**
 * Displays a panel of general options
 * 
 * @author Jeff Michaud, Chris Callendar
 */
public class GeneralOptionsPanel extends JPanel implements ShrimpOptions {
	
	private final int ROW_HEIGHT = 24;
	
	private Properties properties;
	private Hashtable checkBoxes;
	
	private JCheckBox chkbxManyChildren;
	private JIntegerTextField txtManyChildren;
	private JPanel pnlManyChildren;
	private JLabel lblAnimThreshold;
	private JIntegerTextField txtAnimThreshold;
	private JCheckBox chkbxAnim;
	
	private JSpinner spinnerDisplacement;
	private JSpinner spinnerRadius;
	private JSpinner spinnerSpeed;
	private JSpinner spinnerTime;
	private JCheckBox chkbxMotion;
	private ShrimpProject project;
	
	private int rows = 0;
	
    /** Creates new form GeneralOptions */
    public GeneralOptionsPanel(ShrimpProject project) {
    	this.project = project;
		properties = ApplicationAccessor.getApplication().getProperties();
        initComponents();
    }
    
    private void initComponents() {
    	checkBoxes = new Hashtable();
    	
        GridLayout layout = new GridLayout(0, 1, 0, 0);
		setLayout(layout);

    	rows = 0;
            	
        // too many children warning
		initTooManyChildrenOptions();
		initAnimationOptions();
		initMotionOptions();

        setPreferredSize(new Dimension(500, rows * ROW_HEIGHT));
    }
	
	public Component add(Component comp) {
		rows++;
		return super.add(comp);
	}
	
	private JLabel createSpacer(int width) {
    	JLabel lbl = new JLabel(" ");
    	Dimension dim = new Dimension(width, 10);
		lbl.setMinimumSize(dim);
		lbl.setPreferredSize(dim);
    	lbl.setMaximumSize(dim);
    	return lbl;
	}
	
	private void initTooManyChildrenOptions() throws NumberFormatException {
		String thresholdStr = properties.getProperty(DisplayBean.PROPERTY_KEY__SHOW_MANY_CHILDREN_WARNING_THRESHOLD, DisplayBean.DEFAULT_SHOW_MANY_CHILDREN_WARNING_THRESHOLD);
		txtManyChildren = new JIntegerTextField ((new Integer (thresholdStr)).intValue());
		txtManyChildren.setPreferredSize(new Dimension(36, 20));
		
		// too many children warning threshold enable/disable
		String showWarningStr = properties.getProperty(DisplayBean.PROPERTY_KEY__SHOW_MANY_CHILDREN_WARNING, DisplayBean.DEFAULT_SHOW_MANY_CHILDREN_WARNING);
		boolean showWarning = (new Boolean (showWarningStr)).booleanValue();
		chkbxManyChildren = new JCheckBox ("Display warning if opening a node with number of children greater than: ");
		chkbxManyChildren.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				txtManyChildren.setEnabled(chkbxManyChildren.isSelected());
			}
		});
		chkbxManyChildren.setSelected(showWarning);
		
		pnlManyChildren = new JPanel ();
		pnlManyChildren.setLayout(new FlowLayout (FlowLayout.LEFT, 0, 0));
		pnlManyChildren.add(chkbxManyChildren);
		pnlManyChildren.add(txtManyChildren);
		
		add(pnlManyChildren);
	}
	
	private void initAnimationOptions() throws NumberFormatException {
		// animation threshold
		
		lblAnimThreshold = new JLabel("Disable animation if number of nodes greater than: ", JLabel.LEFT);
		String thresholdStr = properties.getProperty(DisplayBean.PROPERTY_KEY__ANIMATION_THRESHOLD, DisplayBean.DEFAULT_ANIMATION_THRESHOLD);
		txtAnimThreshold = new JIntegerTextField((new Integer(thresholdStr)).intValue());
		txtAnimThreshold.setPreferredSize(new Dimension(36, 20));
		
		JPanel pnlAnimThreshold = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		pnlAnimThreshold.add(createSpacer(22));
		pnlAnimThreshold.add(lblAnimThreshold);
		pnlAnimThreshold.add(txtAnimThreshold);
		
		// animation enable/disable
		String animateStr = properties.getProperty(DisplayBean.PROPERTY_KEY__USE_ANIMATION, "" + DisplayBean.DEFAULT_USING_ANIMATION);
		boolean animate = (new Boolean (animateStr)).booleanValue();
		chkbxAnim = new JCheckBox ("Use animation for layouts");
		chkbxAnim.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				txtAnimThreshold.setEnabled(chkbxAnim.isSelected());
				lblAnimThreshold.setEnabled(chkbxAnim.isSelected());
			}
		});
		chkbxAnim.setSelected(animate);
		
		add(chkbxAnim);
		add(pnlAnimThreshold);
	}
	
	private void initMotionOptions() {
		// load prefs
		int displacement = MotionLayout.DEFAULT_DISPLACEMENT;
		double radius = MotionLayoutAlgorithm.DEFAULT_RADIUS;
		int speed = MotionLayout.DEFAULT_SPEED;
		long time = MotionLayoutAlgorithm.DEFAULT_TIME;
		try {
			displacement = Integer.parseInt(properties.getProperty(DisplayBean.PROPERTY_KEY__DISPLACEMENT, DisplayBean.DEFAULT_DISPLACEMENT));
		} catch (NumberFormatException ignore) {}
		try {
			radius = Double.parseDouble(properties.getProperty(DisplayBean.PROPERTY_KEY__MOTION_RADIUS, DisplayBean.DEFAULT_MOTION_RADIUS));
		} catch (NumberFormatException ignore) {}
		try {
			speed = Integer.parseInt(properties.getProperty(DisplayBean.PROPERTY_KEY__MOTION_SPEED, DisplayBean.DEFAULT_MOTION_SPEED));
		} catch (NumberFormatException ignore) {}
		try {
			time = Long.parseLong(properties.getProperty(DisplayBean.PROPERTY_KEY__MOTION_TIME, DisplayBean.DEFAULT_MOTION_TIME));
		} catch (NumberFormatException ignore) {}
		
		
		boolean useMotion = "true".equals(properties.getProperty(DisplayBean.PROPERTY_KEY__USE_MOTION, "true"));
		chkbxMotion = new JCheckBox("Use motion to highlight the neighborhoods of selected nodes by pressing the Alt key", useMotion);

		JPanel pnl1 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		String ttip = "The displacement from the selected node (between 0 and 5) - the higher the displacement value the more nodes moved";
		final JLabel lbl = createLabel("Displacement: ", useMotion, ttip);
		// only allow displacements of [0 ... 5]
		spinnerDisplacement = createSpinner(displacement, 0, 5, 1, useMotion, ttip);
		pnl1.add(createSpacer(22));
		pnl1.add(lbl);
		pnl1.add(spinnerDisplacement);
		
		ttip = "The radius of the motion between 0.1 and 10";
		final JLabel lbl2 = createLabel("Motion radius: ", useMotion, ttip);
		// only allow radius between [0.1 ... 10]
		spinnerRadius = createSpinner(radius, 0.1, 10, 1, useMotion, ttip);
		pnl1.add(createSpacer(22));
		pnl1.add(lbl2);
		pnl1.add(spinnerRadius);

		JPanel pnl2 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		ttip = "The speed of the motion between 1 and 10";
		final JLabel lbl3 = createLabel("Motion speed: ", useMotion, ttip);
		// only allow speed between [0 ... 10]
		spinnerSpeed = createSpinner(speed, 0, 10, 1, useMotion, ttip);
		pnl2.add(createSpacer(22));
		pnl2.add(lbl3);
		pnl2.add(spinnerSpeed);
		
		ttip = "The duration of the motion in milliseconds between 100 and 10000";
		final JLabel lbl4 = createLabel("Motion time (millis): ", useMotion, ttip);
		// only allow time between [100 ... 10000]
		spinnerTime = createSpinner(time, 100, 10000, 500, useMotion, ttip);
		pnl2.add(createSpacer(22));
		pnl2.add(lbl4);
		pnl2.add(spinnerTime);
				
		add(chkbxMotion);
		add(pnl1);
		add(pnl2);
		
		chkbxMotion.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boolean sel = chkbxMotion.isSelected();
				lbl.setEnabled(sel);
				spinnerDisplacement.setEnabled(sel);
				lbl2.setEnabled(sel);
				spinnerRadius.setEnabled(sel);
				lbl3.setEnabled(sel);
				spinnerSpeed.setEnabled(sel);
				lbl4.setEnabled(sel);
				spinnerTime.setEnabled(sel);
			}
		});
	}
	
	private JLabel createLabel(String text, boolean enabled, String ttip) {
		JLabel label = new JLabel(text + " ", JLabel.RIGHT);
		label.setEnabled(enabled);
		label.setToolTipText(ttip);
		label.setPreferredSize(new Dimension(120, 20));
		return label;
	}
	
	private JSpinner createSpinner(double val, double min, double max, double step, boolean enabled, String tooltip) {
		SpinnerNumberModel model = new SpinnerNumberModel(val, min, max, step);
		JSpinner spinner = new JSpinner(model);
		spinner.setEnabled(enabled);
		spinner.setToolTipText(tooltip);
		int size = Double.toString(max).length();
		int width = size * 8;
		spinner.setPreferredSize(new Dimension(Math.max(35, width), 20));
		return spinner;
	}
	
	private int getSpinnerIntValue(JSpinner spinner) {
		SpinnerNumberModel model = (SpinnerNumberModel)spinner.getModel();
		return model.getNumber().intValue();
	}

	private double getSpinnerDoubleValue(JSpinner spinner) {
		SpinnerNumberModel model = (SpinnerNumberModel)spinner.getModel();
		return model.getNumber().doubleValue();
	}
	    
	public void addCheckBoxAction(CheckBoxAction checkBoxAction) {
		JCheckBox checkBox = new JCheckBox(checkBoxAction.getActionName());
		checkBoxes.put(checkBox, checkBoxAction);
		add(checkBox);
		checkBox.setSelected(checkBoxAction.getChecked());
	}    

	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#cancel()
	 */
	public void cancel() {
	}

	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#ok()
	 */
	public void ok() {
		Enumeration e = checkBoxes.keys();
		while (e.hasMoreElements()) {
			JCheckBox checkBox = (JCheckBox) e.nextElement();
			CheckBoxAction checkBoxAction = (CheckBoxAction) checkBoxes.get(checkBox);
			checkBoxAction.setChecked(checkBox.isSelected());
			checkBoxAction.startAction();
		}
		Properties properties = ApplicationAccessor.getApplication().getProperties();
		boolean showWarning = chkbxManyChildren.isSelected();
		properties.setProperty(DisplayBean.PROPERTY_KEY__SHOW_MANY_CHILDREN_WARNING, "" + showWarning);
		int warningThreshold = txtManyChildren.getIntegerText();
		properties.setProperty(DisplayBean.PROPERTY_KEY__SHOW_MANY_CHILDREN_WARNING_THRESHOLD, "" + warningThreshold);
		
		boolean animate = chkbxAnim.isSelected();
		properties.setProperty(DisplayBean.PROPERTY_KEY__USE_ANIMATION, "" + animate);
		int animThreshold = txtAnimThreshold.getIntegerText();
		properties.setProperty(DisplayBean.PROPERTY_KEY__ANIMATION_THRESHOLD, "" + animThreshold);
		
		boolean useMotion = chkbxMotion.isSelected();
		properties.setProperty(DisplayBean.PROPERTY_KEY__USE_MOTION, "" + useMotion);
		int displ = getSpinnerIntValue(spinnerDisplacement);
		properties.setProperty(DisplayBean.PROPERTY_KEY__DISPLACEMENT, Integer.toString(displ));
		double radius = getSpinnerDoubleValue(spinnerRadius);
		properties.setProperty(DisplayBean.PROPERTY_KEY__MOTION_RADIUS, Double.toString(radius));
		int speed = getSpinnerIntValue(spinnerSpeed);
		properties.setProperty(DisplayBean.PROPERTY_KEY__MOTION_SPEED, Integer.toString(speed));
		int time = getSpinnerIntValue(spinnerTime);
		properties.setProperty(DisplayBean.PROPERTY_KEY__MOTION_TIME, Integer.toString(time));
		
		// update the MotionLayout to use the new displacement, radius, speed, and time
		try {
			ShrimpView view = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) view.getBean(ShrimpTool.DISPLAY_BEAN);
			updateMotionLayout(displayBean, displ, radius, speed, time);
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		// update the query view too
		try {
			QueryView queryView = (QueryView)ApplicationAccessor.getApplication().getTool(ShrimpProject.QUERY_VIEW);
			DisplayBean displayBean = (DisplayBean) queryView.getBean(ShrimpTool.DISPLAY_BEAN);
			updateMotionLayout(displayBean, displ, radius, speed, time);
		} catch (Exception ignore) {
		}
	}

	private void updateMotionLayout(DisplayBean displayBean, int displacement, double radius, int speed, long time) {
		Vector v = displayBean.getLayouts();
		for (Iterator iter = v.iterator(); iter.hasNext(); ) {
			Layout layout = (Layout)iter.next();
			if (layout instanceof MotionLayout) {
				MotionLayout motionLayout = ((MotionLayout)layout);
				motionLayout.setDisplacement(displacement);
				motionLayout.setMotionRadius(radius);
				motionLayout.setMotionSpeed(speed);	// set speed BEFORE time
				motionLayout.setMotionTime(time);
				break;	// there can't be more than one right?
			}
		}
	}
}
